@@ -18,6 +18,8 @@
 
 #define G_LOG_DOMAIN "jsonrpc-client"
 
+#include "config.h"
+
 /**
  * SECTION:jsonrpc-client
  * @title: JsonrpcClient
@@ -240,6 +242,30 @@ error_invocations_from_idle (gpointer data)
   return G_SOURCE_REMOVE;
 }
 
+static void
+cancel_pending_from_main (JsonrpcClient *self,
+                          const GError  *error)
+{
+  JsonrpcClientPrivate *priv = jsonrpc_client_get_instance_private (self);
+  PanicData *pd;
+
+  g_assert (JSONRPC_IS_CLIENT (self));
+  g_assert (error != NULL);
+
+  /*
+   * Defer the completion of all tasks (with errors) until we've made it
+   * back to the main loop. Otherwise, we get into difficult to determine
+   * re-entrancy cases.
+   */
+  pd = g_slice_new0 (PanicData);
+  pd->invocations = g_steal_pointer (&priv->invocations);
+  pd->error = g_error_copy (error);
+  g_idle_add_full (G_MAXINT, error_invocations_from_idle, pd, NULL);
+
+  /* Keep a hashtable around for code that expects a pointer there */
+  priv->invocations = g_hash_table_new_full (NULL, NULL, NULL, g_object_unref);
+}
+
 /*
  * jsonrpc_client_panic:
  *
@@ -252,7 +278,6 @@ jsonrpc_client_panic (JsonrpcClient *self,
 {
   JsonrpcClientPrivate *priv = jsonrpc_client_get_instance_private (self);
   g_autoptr(JsonrpcClient) hold = NULL;
-  PanicData *pd;
 
   g_assert (JSONRPC_IS_CLIENT (self));
   g_assert (error != NULL);
@@ -261,18 +286,7 @@ jsonrpc_client_panic (JsonrpcClient *self,
 
   priv->failed = TRUE;
 
-  /*
-   * Defer the completion of all tasks (with errors) until we've made it
-   * back to the main loop. Otherwise, we get into difficult to determine
-   * re-entrancy cases.
-   */
-  pd = g_slice_new0 (PanicData);
-  pd->invocations = g_steal_pointer (&priv->invocations);
-  pd->error = g_error_copy (error);
-  g_idle_add_full (G_PRIORITY_LOW, error_invocations_from_idle, pd, NULL);
-
-  /* Keep a hashtable around for code that expects a pointer there */
-  priv->invocations = g_hash_table_new_full (NULL, NULL, NULL, g_object_unref);
+  cancel_pending_from_main (self, error);
 
   /* Now close the connection */
   jsonrpc_client_close (self, NULL, NULL);
@@ -283,8 +297,6 @@ jsonrpc_client_panic (JsonrpcClient *self,
    */
   g_clear_object (&priv->input_stream);
   g_clear_object (&priv->output_stream);
-
-  g_signal_emit (hold, signals [FAILED], 0);
 }
 
 /*
@@ -560,7 +572,7 @@ jsonrpc_client_new (GIOStream *io_stream)
 }
 
 static void
-jsonrpc_client_remove_from_invocatoins (JsonrpcClient *self,
+jsonrpc_client_remove_from_invocations (JsonrpcClient *self,
                                         GTask         *task)
 {
   JsonrpcClientPrivate *priv = jsonrpc_client_get_instance_private (self);
@@ -584,7 +596,7 @@ jsonrpc_client_call_notify_completed (JsonrpcClient *self,
   g_assert (g_str_equal (pspec->name, "completed"));
   g_assert (G_IS_TASK (task));
 
-  jsonrpc_client_remove_from_invocatoins (self, task);
+  jsonrpc_client_remove_from_invocations (self, task);
 }
 
 static void
@@ -606,15 +618,14 @@ jsonrpc_client_call_write_cb (GObject      *object,
 
   if (!jsonrpc_output_stream_write_message_finish (stream, result, &error))
     {
-      jsonrpc_client_remove_from_invocatoins (self, task);
+      /* Panic will cancel our task, no need to return error here */
       jsonrpc_client_panic (self, error);
-      g_task_return_error (task, g_steal_pointer (&error));
       return;
     }
 
   /* We don't need to complete the task because it will get completed when the
-   * server replies with our reply. This is performed using an asynchronous
-   * read that will pump through the messages.
+   * server replies with our return value. This is performed using an
+   * asynchronous read that will pump through incoming messages.
    */
 }
 
@@ -636,11 +647,9 @@ jsonrpc_client_call_read_cb (GObject      *object,
   if (!jsonrpc_input_stream_read_message_finish (stream, result, &message, &error))
     {
       /* Handle jsonrpc_client_close() conditions gracefully. */
-      if (priv->in_shutdown && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-        {
-          g_io_stream_close (priv->io_stream, NULL, NULL);
-          return;
-        }
+      if (priv->in_shutdown &&
+          g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+        return;
 
       /*
        * If we fail to read a message, that means we couldn't even receive
@@ -660,6 +669,9 @@ jsonrpc_client_call_read_cb (GObject      *object,
   /* Make sure we got a proper type back from the variant. */
   if (!g_variant_is_of_type (message, G_VARIANT_TYPE_VARDICT))
     {
+      error = g_error_new_literal (G_IO_ERROR,
+                                   G_IO_ERROR_INVALID_DATA,
+                                   "Improper reply from peer, not a vardict");
       jsonrpc_client_panic (self, error);
       return;
     }
@@ -675,7 +687,7 @@ jsonrpc_client_call_read_cb (GObject      *object,
     {
       error = g_error_new_literal (G_IO_ERROR,
                                    G_IO_ERROR_INVALID_DATA,
-                                   "Received malformed response from peer");
+                                   "Improper reply from peer");
       jsonrpc_client_panic (self, error);
       return;
     }
@@ -738,9 +750,9 @@ jsonrpc_client_call_read_cb (GObject      *object,
       if (!g_variant_dict_lookup (&dict, "method", "&s", &method_name) ||
           NULL == (id = g_variant_dict_lookup_value (&dict, "id", NULL)))
         {
-          error = g_error_new (G_IO_ERROR,
-                               G_IO_ERROR_INVALID_DATA,
-                               "Call contains invalid method or id field");
+          error = g_error_new_literal (G_IO_ERROR,
+                                       G_IO_ERROR_INVALID_DATA,
+                                       "Call contains invalid method or id field");
           jsonrpc_client_panic (self, error);
           return;
         }
@@ -1190,6 +1202,7 @@ jsonrpc_client_close (JsonrpcClient  *self,
 {
   JsonrpcClientPrivate *priv = jsonrpc_client_get_instance_private (self);
   g_autoptr(GHashTable) invocations = NULL;
+  g_autoptr(GError) local_error = NULL;
   gboolean ret;
 
   g_return_val_if_fail (JSONRPC_IS_CLIENT (self), FALSE);
@@ -1212,24 +1225,10 @@ jsonrpc_client_close (JsonrpcClient  *self,
    * Closing the input stream will fail, so just rely on the callback
    * from the async function to complete/close the stream.
    */
-
-  invocations = g_steal_pointer (&priv->invocations);
-  priv->invocations = g_hash_table_new_full (NULL, NULL, NULL, g_object_unref);
-
-  if (g_hash_table_size (invocations) > 0)
-    {
-      g_autoptr(GError) local_error = NULL;
-      GTask *task;
-      GHashTableIter iter;
-
-      local_error = g_error_new_literal (G_IO_ERROR,
-                                         G_IO_ERROR_CLOSED,
-                                         "The underlying stream was closed");
-
-      g_hash_table_iter_init (&iter, invocations);
-      while (g_hash_table_iter_next (&iter, NULL, (gpointer *)&task))
-        g_task_return_error (task, g_error_copy (local_error));
-    }
+  local_error = g_error_new_literal (G_IO_ERROR,
+                                     G_IO_ERROR_CLOSED,
+                                     "The underlying stream was closed");
+  cancel_pending_from_main (self, local_error);
 
   g_signal_emit (self, signals [FAILED], 0);
 
